@@ -60,7 +60,6 @@
 #include "imports.h"
 #include "context.h"
 #include "macros.h"
-#include "mfeatures.h"
 #include "get.h"
 #include "dispatch.h"
 #include "mtypes.h"
@@ -410,7 +409,7 @@ _mesa_GetSynciv(GLsync sync, GLenum pname, GLsizei bufSize, GLsizei *length,
       return;
    }
 
-   if (size > 0) {
+   if (size > 0 && bufSize > 0) {
       const GLsizei copy_count = MIN2(size, bufSize);
 
       memcpy(values, v, sizeof(GLint) * copy_count);
